# WARNING - Generated by {fusen} from /dev/flat_full.Rmd: do not edit by hand


#' Get Dispersion Comparison
#'
#' Compares inferred dispersion values with actual dispersion values.
#'
#' @param inferred_dispersion A data frame containing inferred dispersion values.
#' @param actual_dispersion A numeric vector containing actual dispersion values.
#'
#' @return A data frame comparing actual and inferred dispersion values.
#' 
#' @export
#'
#' @examples
#' \dontrun{
#' dispersion_comparison <- getDispersionComparison(inferred_disp, actual_disp)
#' }
getDispersionComparison <- function(inferred_dispersion, actual_dispersion) {
  actual_disp <- data.frame(actual = actual_dispersion)
  actual_disp$ID <- rownames(actual_disp)
  rownames(actual_disp) <- NULL
  disp_comparison <- join_dtf(actual_disp, inferred_dispersion, c("ID"), c("ID"))
  disp_comparison$term <- 'dispersion'
  disp_comparison$description <- 'dispersion'
  return(disp_comparison)
}


#' Extract DESeq2 Dispersion Values
#'
#' Extracts inferred dispersion values from a DESeq2 wrapped object.
#'
#' @param dds_wrapped A DESeq2 wrapped object containing dispersion values.
#'
#' @return A data frame containing inferred dispersion values.
#' 
#' @export
#'
#' @examples
#' \dontrun{
#' dispersion_df <- extract_ddsDispersion(deseq2_object)
#' }
extract_ddsDispersion <- function(dds_wrapped) {
  inferred_dispersion <- data.frame(estimate = dds_wrapped$dispersion)
  inferred_dispersion$ID <- rownames(inferred_dispersion)
  rownames(inferred_dispersion) <- NULL
  return(inferred_dispersion)
}


#' Extract TMB Dispersion Values
#'
#' Extracts inferred dispersion values from a TMB result object.
#'
#' @param list_tmb A TMB result object containing dispersion values.
#'
#' @return A data frame containing inferred dispersion values.
#' 
#' @export
#'
#' @examples
#' \dontrun{
#' dispersion_df <- extract_tmbDispersion(tmb_result)
#' }
extract_tmbDispersion <- function(list_tmb) {
  glanceRes <- glance_tmb(list_tmb)
  inferred_dispersion <- data.frame(estimate = glanceRes$dispersion)
  inferred_dispersion$ID <- rownames(glanceRes)
  rownames(inferred_dispersion) <- NULL
  return(inferred_dispersion)
}


