# WARNING - Generated by {fusen} from /dev/flat_full.Rmd: do not edit by hand


#' Subset Genes in Genomic Data
#'
#' This function filters and adjusts genomic data within the Roxygeb project, based on a specified list of genes.
# It is designed to enhance precision and customization in transcriptomics analysis by retaining only the genes of interest.
# 
#' @param l_genes A character vector specifying the genes to be retained in the dataset.
#' @param mockObj An object containing relevant genomic information to be filtered.
#'
#' @return A modified version of the 'mockObj' data object, with genes filtered according to 'l_genes'.
#'
#' @description The 'subsetGenes' function selects and retains genes from 'mockObj' that match the genes specified in 'l_genes'.
# It filters the 'groundTruth$effects' data to keep only the rows corresponding to the selected genes. 
# Additionally, it updates 'gene_dispersion' and the count data, ensuring that only the selected genes are retained.
# The function also replaces the total number of genes in 'settings$values' with the length of 'l_genes'.
# The result is a more focused and tailored genomic dataset, facilitating precision in subsequent analyses.
#'
#' @examples
#' \dontrun{
#' # Example list of genes to be retained
#' selected_genes <- c("GeneA", "GeneB", "GeneC")
#'
#' # Example data object 'mockObj' (simplified structure)
#' mockObj <- list(
#'   # ... (mockObj structure)
#' )
#'
#' # Using the subsetGenes function to filter 'mockObj'
#' filtered_mockObj <- subsetGenes(selected_genes, mockObj)
#' }
#' @export
subsetGenes <- function(l_genes, mockObj) {
  # Selects the indices of genes in 'groundTruth$effects$geneID' that are present in 'l_genes'.
  idx_gt_effects <- mockObj$groundTruth$effects$geneID %in% l_genes
  
  # Filters 'groundTruth$effects' to keep only the rows corresponding to the selected genes.
  mockObj$groundTruth$effects <- mockObj$groundTruth$effects[idx_gt_effects, ]
  
  # Updates 'gene_dispersion' by retaining values corresponding to the selected genes.
  mockObj$groundTruth$gene_dispersion <- mockObj$groundTruth$gene_dispersion[l_genes]
  
  # Filters the count data to keep only the rows corresponding to the selected genes.
  mockObj$counts <- as.data.frame(mockObj$counts[l_genes, ])
  
  # Replaces the total number of genes in 'settings$values' with the length of 'l_genes'.
  mockObj$settings$values[1] <- length(l_genes)
  
  # Returns the modified 'mockObj'.
  return(mockObj)
}


